import { StorageApi } from '../util/storage-api';
import { HttpResponse } from '@angular/common/http';
import { NetworkService } from '../util/network';
import { Observable } from 'rxjs';
import { DaoContract } from './dao-contract';
import { Id } from '../model/key/id';
import { Entity } from '../model/entity';
import { PatchUpdate } from '../model/patch-update';
import { QueryParams } from '../util/query-params';
import { PaginatedResult } from '../util/paginated-result';
export declare abstract class Dao<T extends Entity> implements DaoContract<T> {
    protected api: StorageApi;
    protected network: NetworkService;
    baseApiUrl: string;
    readonly API_URL: string;
    abstract deserialize(response: HttpResponse<T>): HttpResponse<T>;
    abstract deserializeMany(response: HttpResponse<T[]>): HttpResponse<T[]>;
    private qs;
    constructor(api: StorageApi, network: NetworkService, apiPath: string);
    protected isOnline(): boolean;
    private response;
    private responseMany;
    withDefaultHeaders(): {
        headers?: {
            [header: string]: string | string[];
        };
        observe: 'response';
    };
    create(t: T): Observable<HttpResponse<T>>;
    createMany(xs: T[]): Observable<HttpResponse<T[]>>;
    update(t: T): Observable<HttpResponse<T>>;
    updateMany(xs: PatchUpdate[]): Observable<HttpResponse<PatchUpdate[]>>;
    private patchLocal;
    delete(t: T): Observable<HttpResponse<T>>;
    deleteMany(xs: T[]): Observable<HttpResponse<T[]>>;
    findById<Key extends Id>(id: Key): Observable<HttpResponse<T>>;
    findAll(q?: QueryParams): Observable<HttpResponse<PaginatedResult<T>>>;
    protected paginate(x: HttpResponse<T[]>, totalRecords?: number, currentPage?: number): HttpResponse<PaginatedResult<T>>;
    protected combineMany(payload: Observable<HttpResponse<T[]>>): Observable<HttpResponse<T[]>>;
    private getManyLocal;
    private getLocal;
    private setLocal;
    private unsetLocal;
}
