import { EntityAdapter } from '@ngrx/entity';
import { EntityState } from './state';
import { Entity } from '../model/entity';
import { Id } from '../model/key/id';
import { PatchUpdate } from '../model/patch-update';
import { PaginatedResult } from '../util/paginated-result';
export declare class EntityReducer<T extends Entity, Key extends Id, S extends EntityState<T, Key>> {
    readonly adapter: EntityAdapter<T>;
    readonly state: S;
    protected serialize: (t: T) => T;
    constructor(adapter: EntityAdapter<T>, state: S, serialize: (t: T) => T);
    private currentPage;
    private totalRecords;
    private getDefaultState;
    loading(): S & {
        isLoading: boolean;
        totalRecords: number;
        currentPage: number;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    loaded(x: PaginatedResult<T>): S & {
        isLoaded: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    loadedOne(payload: T): S & {
        isLoaded: boolean;
        selected: T;
        selectedId: Id<string>;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        error: any;
    };
    add(): S & {
        isAdding: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    addMany(): S & {
        isAdding: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    private setAdding;
    added(payload: T): S & {
        isAdded: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    } & {
        selected: T;
        selectedId: Id<string>;
    };
    addedMany(payload: T[]): S & {
        isAdded: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    private addedState;
    update(): S & {
        isUpdating: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    updateMany(): S & {
        isUpdating: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    private setUpdating;
    updated(payload: T): S & {
        selected: T;
        selectedId: Id<string>;
        isUpdated: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        error: any;
    };
    updatedMany(payload: PatchUpdate[]): S;
    delete(): S & {
        isDeleting: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    deleteMany(): S & {
        isDeleting: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    private setDeleting;
    deleted(payload: T): S & {
        isDeleted: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    deletedMany(payload: T[]): S & {
        isDeleted: boolean;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        selected: any;
        selectedId: any;
        error: any;
    };
    private deletedState;
    selected(payload?: T): S & {
        selected: T;
        selectedId: Id<string>;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        error: any;
    };
    failed(payload: Error): S & {
        error: string;
        totalRecords: number;
        currentPage: number;
        isLoading: boolean;
        isLoaded: boolean;
        isAdding: boolean;
        isAdded: boolean;
        isUpdating: boolean;
        isUpdated: boolean;
        isDeleting: boolean;
        isDeleted: boolean;
        selected: any;
        selectedId: any;
    };
    private setTotalRecords;
}
export declare class EntitySelector<T extends Entity> {
    readonly adapter: EntityAdapter<T>;
    constructor(adapter: EntityAdapter<T>);
    selectors(): {
        selectIds: (state: import("@ngrx/entity").EntityState<T>) => string[] | number[];
        selectEntities: (state: import("@ngrx/entity").EntityState<T>) => import("@ngrx/entity").Dictionary<T>;
        selectAll: (state: import("@ngrx/entity").EntityState<T>) => T[];
        selectTotal: (state: import("@ngrx/entity").EntityState<T>) => number;
    };
}
