import { defaultState } from './state';
export class EntityReducer {
    constructor(adapter, state, serialize) {
        this.adapter = adapter;
        this.state = state;
        this.serialize = serialize;
        this.currentPage = this.state.currentPage;
        this.totalRecords = this.state.totalRecords;
    }
    getDefaultState() {
        return {
            ...defaultState,
            totalRecords: this.totalRecords,
            currentPage: this.currentPage
        };
    }
    loading() {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isLoading: true
        });
    }
    loaded(x) {
        this.currentPage = x.currentPage;
        if (x.totalRecords > this.state.totalRecords) {
            this.totalRecords = x.totalRecords;
        }
        const state = Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isLoaded: true
        });
        return this.adapter.upsertMany(x.payload, state);
    }
    loadedOne(payload) {
        const state = Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isLoaded: true,
            selected: payload,
            selectedId: payload.id
        });
        return this.adapter.upsertOne(payload, state);
    }
    add() {
        return this.setAdding();
    }
    addMany() {
        return this.setAdding();
    }
    setAdding() {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isAdding: true
        });
    }
    added(payload) {
        const state = Object.assign({}, this.addedState(1), {
            selected: payload,
            selectedId: payload.id
        });
        return this.adapter.upsertOne(payload, state);
    }
    addedMany(payload) {
        return this.adapter.addMany(payload, this.addedState(payload === null || payload === void 0 ? void 0 : payload.length));
    }
    addedState(count) {
        this.setTotalRecords('add', count);
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isAdded: true
        });
    }
    update() {
        return this.setUpdating();
    }
    updateMany() {
        return this.setUpdating();
    }
    setUpdating() {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isUpdating: true
        });
    }
    updated(payload) {
        return this.adapter.updateOne({ id: `${payload.id}`, changes: payload }, Object.assign({}, this.state, {
            ...this.getDefaultState(),
            selected: payload,
            selectedId: payload.id,
            isUpdated: true
        }));
    }
    updatedMany(payload) {
        let entities = [];
        this.adapter
            .getSelectors()
            .selectAll(this.state)
            .map(x => this.serialize(x))
            .forEach(entity => {
            const partial = payload.find(p => p.id === entity.id);
            if (partial) {
                entities.push(entity.copy({ ...partial.params }));
            }
        });
        if (entities && entities.length) {
            return this.adapter.updateMany(entities.map(x => {
                return { id: `${x.id}`, changes: x };
            }), Object.assign({}, this.state, {
                ...this.getDefaultState(),
                isUpdated: true
            }));
        }
        else
            return this.state;
    }
    delete() {
        return this.setDeleting();
    }
    deleteMany() {
        return this.setDeleting();
    }
    setDeleting() {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            isDeleting: true
        });
    }
    deleted(payload) {
        const state = this.deletedState(this.state, 1);
        return this.adapter.removeOne(`${payload.id}`, state);
    }
    deletedMany(payload) {
        const state = this.deletedState(this.state, payload.length);
        const ids = payload.map(x => `${x.id}`);
        return this.adapter.removeMany(ids, state);
    }
    deletedState(state, count) {
        this.setTotalRecords('minus', count);
        return Object.assign({}, state, {
            ...this.getDefaultState(),
            isDeleted: true
        });
    }
    selected(payload) {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            selected: payload,
            selectedId: payload ? payload.id : null
        });
    }
    failed(payload) {
        return Object.assign({}, this.state, {
            ...this.getDefaultState(),
            error: payload.message
        });
    }
    setTotalRecords(op, count) {
        const x = this.state.totalRecords;
        if (x)
            this.totalRecords = op === 'add' ? x + count : x - count;
    }
}
export class EntitySelector {
    constructor(adapter) {
        this.adapter = adapter;
    }
    selectors() {
        const { selectIds: selectIds, selectEntities: selectEntities, selectAll: selectAll, selectTotal: selectTotal } = this.adapter.getSelectors();
        return {
            selectIds: selectIds,
            selectEntities: selectEntities,
            selectAll: selectAll,
            selectTotal: selectTotal
        };
    }
}
//# sourceMappingURL=reducer.js.map