import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { Entity } from '../model/entity';
import { EntityState } from './state';
import { EntityActions } from './action';
import { Id } from '../model/key/id';
import { PatchUpdate } from '../model/patch-update';
import { QueryParams } from '../util/query-params';
import { PaginatedResult } from '../util/paginated-result';
export declare abstract class EntityStore<T extends Entity, Key extends Id> {
    protected entity: EntityActions<T, Key>;
    protected store$: Observable<EntityState<T, Key>>;
    protected selectAll: (state: EntityState<T, Key>) => T[];
    protected store: Store<any>;
    loading$: Observable<boolean>;
    loaded$: Observable<boolean>;
    adding$: Observable<boolean>;
    added$: Observable<boolean>;
    updating$: Observable<boolean>;
    updated$: Observable<boolean>;
    deleting$: Observable<boolean>;
    deleted$: Observable<boolean>;
    selected$: Observable<T>;
    entities$: Observable<T[]>;
    paginated$: Observable<PaginatedResult<T>>;
    success$: Observable<T[]>;
    error$: Observable<any>;
    constructor(entity: EntityActions<T, Key>, store$: Observable<EntityState<T, Key>>, selectAll: (state: EntityState<T, Key>) => T[]);
    onResult(success: (t: T | T[]) => void, error?: (t: any) => void): void;
    onSuccess(op: (t: T | T[]) => void): void;
    onError(op: (t: any) => void): void;
    private dispatch;
    load(q?: QueryParams): void;
    loadOne(id: Key): Observable<T>;
    select(x: T): void;
    create(x: T): void;
    createMany(xs: T[]): void;
    update(x: T): void;
    updateMany(xs: PatchUpdate[]): void;
    delete(x: T): void;
    deleteMany(xs: T[]): void;
    findBy(cond: (u: T) => boolean): Observable<T>;
    findAllBy(cond: (u: T) => boolean): Observable<T[]>;
}
