import { combineLatest } from 'rxjs';
import { map, filter, take } from 'rxjs/operators';
export class EntityStore {
    constructor(entity, store$, selectAll) {
        this.entity = entity;
        this.store$ = store$;
        this.selectAll = selectAll;
        this.entities$ = store$.pipe(map(x => selectAll(x)));
        this.loading$ = store$.pipe(map(x => x.isLoading));
        this.loaded$ = store$.pipe(map(x => x.isLoaded));
        this.adding$ = store$.pipe(map(x => x.isAdding));
        this.added$ = store$.pipe(map(x => x.isAdded));
        this.updating$ = store$.pipe(map(x => x.isUpdating));
        this.updated$ = store$.pipe(map(x => x.isUpdated));
        this.deleting$ = store$.pipe(map(x => x.isDeleting));
        this.deleted$ = store$.pipe(map(x => x.isDeleted));
        this.selected$ = store$.pipe(map(x => x.selected));
        this.paginated$ = combineLatest(this.entities$, store$.pipe(map(x => x.totalRecords)), store$.pipe(map(x => x.currentPage))).pipe(map(([xs, total, page]) => {
            return { payload: xs, totalRecords: total, currentPage: page };
        }));
        this.success$ = combineLatest(this.entities$, this.selected$, this.added$, this.updated$, this.deleted$).pipe(filter(([entities, selected, a, b, deleted]) => (a || b || deleted) &&
            (Boolean(selected) || deleted || (entities === null || entities === void 0 ? void 0 : entities.length) > 0)), map(([entities, selected, , ,]) => {
            if (selected)
                return [selected];
            else
                return entities;
        }));
        this.error$ = store$.pipe(map(x => x.error));
    }
    onResult(success, error) {
        this.onSuccess(success);
        if (error)
            this.onError(error);
    }
    onSuccess(op) {
        this.success$.pipe(take(1)).subscribe(xs => {
            if ((xs === null || xs === void 0 ? void 0 : xs.length) === 1)
                op(xs[0]);
            else
                op(xs);
        });
    }
    onError(op) {
        this.error$
            .pipe(filter(x => !!x), take(1))
            .subscribe(x => op(x));
    }
    dispatch(action) {
        this.store.dispatch(action);
    }
    load(q) {
        this.dispatch(this.entity.loading(q));
    }
    loadOne(id) {
        this.dispatch(this.entity.loadingOne(id));
        return this.selected$.pipe(filter(x => !!x));
    }
    select(x) {
        this.dispatch(this.entity.select(x));
    }
    create(x) {
        this.dispatch(this.entity.create(x));
    }
    createMany(xs) {
        this.dispatch(this.entity.createMany(xs));
    }
    update(x) {
        this.dispatch(this.entity.update(x));
    }
    updateMany(xs) {
        this.dispatch(this.entity.updateMany(xs));
    }
    delete(x) {
        this.dispatch(this.entity.delete(x));
    }
    deleteMany(xs) {
        this.dispatch(this.entity.deleteMany(xs));
    }
    findBy(cond) {
        return this.entities$.pipe(map(xs => xs && xs.find(t => cond(t))));
    }
    findAllBy(cond) {
        return this.entities$.pipe(map(xs => xs && xs.filter(t => cond(t))));
    }
}
//# sourceMappingURL=store.js.map