import { Injectable } from '@angular/core';
import { of } from 'rxjs';
import { catchError, filter, map, tap } from 'rxjs/operators';
import { webSocket } from 'rxjs/webSocket';
import { Store } from '@ngrx/store';
import * as i0 from "@angular/core";
import * as i1 from "@ngrx/store";
export class Websocket {
    constructor(store, environment) {
        this.store = store;
        this.environment = environment;
        // params for reconnecting websocket
        this.retryMax = 36;
        this.retryCount = 0;
        this.retryInterval = 5000;
        this.retry = (session) => {
            this.retryCount++;
            console.log(`Trying to reconnect: ${this.retryCount}`);
            if (this.retryMax > this.retryCount)
                setTimeout(() => this.connectToSocket(session), this.retryInterval);
            else
                console.error(`RECONNECT FAILED! Gave up after ${this.retryMax} attempts`);
        };
        this.actionRegex = /(ADDED|UPDATED|DELETED)_(.*)/;
    }
    connect(session) {
        if (!this.isConnected()) {
            this.connectToSocket(session);
        }
    }
    disconnect() {
        if (this.isConnected()) {
            this.ws.unsubscribe();
            this.ws = null;
        }
    }
    isConnected() {
        return this.ws && !this.ws.closed;
    }
    connectToSocket(session) {
        if (!this.ws) {
            this.createSocket(session);
            this.subscribeToSocket(session);
        }
    }
    createSocket(x) {
        const url = `${this.environment.wss}/socket/${x.id}`;
        this.ws = webSocket(url);
    }
    subscribeToSocket(session) {
        const keepalive = setInterval(() => this.ping(), 30000);
        const retry = (msg, err) => {
            console.error(`WEBSOCKET ${msg}!`, err || '');
            clearInterval(keepalive);
            this.disconnect();
            this.retry(session);
        };
        this.ws
            .pipe(tap(_ => {
            // socket reconnected
            if (this.retryCount !== 0) {
                console.log('WEBSOCKET CONNECTED');
                this.load(session.user); // sync backend changes since last connect
            }
            this.retryCount = 0;
        }), catchError(_ => of({ type: 'ERROR', payload: {} })), filter(x => this.actionRegex.test(x.type)), map(x => this.getAction(x)))
            .subscribe(x => this.store.dispatch(x), err => retry('ERROR', err), () => retry('CLOSED'));
    }
    ping() {
        this.ws ? this.ws.next({ type: 'PING', payload: {} }) : null;
    }
    getActionType(_type) {
        let action;
        if (_type.startsWith('ADDED_MANY'))
            action = 'ADDED_MANY';
        else if (_type.startsWith('ADDED'))
            action = 'ADDED';
        else if (_type.startsWith('UPDATED_MANY'))
            action = 'UPDATED_MANY';
        else if (_type.startsWith('UPDATED'))
            action = 'UPDATED';
        else if (_type.startsWith('DELETED_MANY'))
            action = 'DELETED_MANY';
        else
            action = 'DELETED';
        return action;
    }
}
Websocket.ɵfac = function Websocket_Factory(t) { i0.ɵɵinvalidFactory(); };
Websocket.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: Websocket, factory: Websocket.ɵfac, providedIn: 'root' });
(function () { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(Websocket, [{
        type: Injectable,
        args: [{ providedIn: 'root' }]
    }], function () { return [{ type: i1.Store }, { type: undefined }]; }, null); })();
//# sourceMappingURL=websocket.js.map